# -*- coding: utf-8 -*-
"""
Utility functions for interfacing with the Django messages framework.
"""
from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import messages
from django.utils.translation import ugettext as _

from enterprise.utils import get_configuration_value


def add_consent_declined_message(request, enterprise_customer, item):
    """
    Add a message to the Django messages store indicating that the user has declined data sharing consent.

    Arguments:
        request (HttpRequest): The current request.
        enterprise_customer (EnterpriseCustomer): The EnterpriseCustomer associated with this request.
        item (str): A string containing information about the item for which consent was declined.
    """
    messages.warning(
        request,
        _(
            '{strong_start}We could not enroll you in {em_start}{item}{em_end}.{strong_end} '
            '{span_start}If you have questions or concerns about sharing your data, please contact your learning '
            'manager at {enterprise_customer_name}, or contact {link_start}{platform_name} support{link_end}.{span_end}'
        ).format(
            item=item,
            em_start='<em>',
            em_end='</em>',
            enterprise_customer_name=enterprise_customer.name,
            link_start='<a href="{support_link}" target="_blank">'.format(
                support_link=get_configuration_value(
                    'ENTERPRISE_SUPPORT_URL',
                    getattr(settings, 'ENTERPRISE_SUPPORT_URL', '')  # Remove the `getattr` when setting is upstreamed.
                ),
            ),
            platform_name=get_configuration_value('PLATFORM_NAME', settings.PLATFORM_NAME),
            link_end='</a>',
            span_start='<span>',
            span_end='</span>',
            strong_start='<strong>',
            strong_end='</strong>',
        )
    )


def add_missing_price_information_message(request, item):
    """
    Add a message to the Django messages store indicating that we failed to retrieve price information about an item.

    :param request: The current request.
    :param item: The item for which price information is missing. Example: a program title, or a course.
    """
    messages.warning(
        request,
        _(
            '{strong_start}We could not gather price information for {em_start}{item}{em_end}.{strong_end} '
            '{span_start}If you continue to have these issues, please contact '
            '{link_start}{platform_name} support{link_end}.{span_end}'
        ).format(
            item=item,
            em_start='<em>',
            em_end='</em>',
            link_start='<a href="{support_link}" target="_blank">'.format(
                support_link=get_configuration_value(
                    'ENTERPRISE_SUPPORT_URL',
                    getattr(settings, 'ENTERPRISE_SUPPORT_URL', '')  # Remove the `getattr` when setting is upstreamed.
                ),
            ),
            platform_name=get_configuration_value('PLATFORM_NAME', settings.PLATFORM_NAME),
            link_end='</a>',
            span_start='<span>',
            span_end='</span>',
            strong_start='<strong>',
            strong_end='</strong>',
        )
    )


def add_not_one_click_purchasable_message(request, enterprise_customer, program_title):
    """
    Add a message to the Django message store indicating that the program is not one-click purchasable.

    Only one-click purchasable programs should be available for enrollment through the Enterprise landing page flow.

    :param request: The current request.
    :param enterprise_customer: The Enterprise Customer to which the program is associated.
    :param program_title: The title of the program that is not one-click purchasable.
    """
    messages.warning(
        request,
        _(
            '{strong_start}We could not load the program titled {em_start}{program_title}{em_end} through '
            '{enterprise_customer_name}.{strong_end} '
            '{span_start}If you have any questions, please contact your learning manager at '
            '{enterprise_customer_name}, or contact {link_start}{platform_name} support{link_end}.{span_end}'
        ).format(
            program_title=program_title,
            enterprise_customer_name=enterprise_customer.name,
            em_start='<em>',
            em_end='</em>',
            link_start='<a href="{support_link}" target="_blank">'.format(
                support_link=get_configuration_value(
                    'ENTERPRISE_SUPPORT_URL',
                    getattr(settings, 'ENTERPRISE_SUPPORT_URL', '')  # Remove the `getattr` when setting is upstreamed.
                ),
            ),
            platform_name=get_configuration_value('PLATFORM_NAME', settings.PLATFORM_NAME),
            link_end='</a>',
            span_start='<span>',
            span_end='</span>',
            strong_start='<strong>',
            strong_end='</strong>',
        )
    )



# -*- coding: utf-8 -*-
"""
Transmits information about an enterprise's course catalog to connected IntegratedChannels
"""

from __future__ import absolute_import, unicode_literals

from logging import getLogger

from django.contrib.auth.models import User
from django.core.management.base import BaseCommand, CommandError

from integrated_channels.sap_success_factors.models import SAPSuccessFactorsEnterpriseCustomerConfiguration
from integrated_channels.sap_success_factors.utils import SapCourseExporter

from . import INTEGRATED_CHANNEL_CHOICES, IntegratedChannelCommandMixin, celery_task


PLUGIN_MAPPING = {
    SAPSuccessFactorsEnterpriseCustomerConfiguration: SapCourseExporter,
}

LOGGER = getLogger(__name__)


class Command(IntegratedChannelCommandMixin, BaseCommand):
    """
    Transmit courseware data to the IntegratedChannel(s) linked to any or all EnterpriseCustomers.
    """

    def add_arguments(self, parser):
        """
        Add required arguments to the parser.
        """
        parser.add_argument(
            '--catalog_user',
            dest='catalog_user',
            required=True,
            metavar='ENTERPRISE_CATALOG_API_USERNAME',
            help='Use this user to access the Course Catalog API.'
        )
        super(Command, self).add_arguments(parser)

    def handle(self, *args, **options):
        """
        Transmit the courseware data for the EnterpriseCustomer(s) to the active integration channels.
        """
        username = options['catalog_user']

        # Before we do a whole bunch of database queries, make sure that the user we were passed exists.
        try:
            User.objects.get(username=username)
        except User.DoesNotExist:
            raise CommandError('A user with the username {} was not found.'.format(username))

        channels = self.get_integrated_channels(options, enterprise_customer__catalog__isnull=False)

        for channel in channels:
            channel_code = channel.channel_code()
            channel_pk = channel.pk
            send_data_task.delay(username, channel_code, channel_pk)


@celery_task
def send_data_task(username, channel_code, channel_pk):
    """
    Task to send course data to each linked integrated channel

    Arguments:
        channel_code (str): Capitalized identifier for the integrated channel
        channel_pk (str): Primary key for identifying integrated channel

    """
    user = User.objects.get(username=username)
    channel = INTEGRATED_CHANNEL_CHOICES[channel_code].objects.get(pk=channel_pk)

    LOGGER.info(
        'Processing courses for integrated channel using configuration: %s',
        channel,
    )

    try:
        channel.transmit_course_data(user)
    except Exception:  # pylint: disable=broad-except
        exception_message = 'Transmission of course metadata failed for user "{username}" and for integrated ' \
                            'channel with code "{channel_code}" and id "{channel_pk}".'.format(
                                username=username,
                                channel_code=channel_code,
                                channel_pk=channel_pk,
                            )
        LOGGER.exception(exception_message)



# -*- coding: utf-8 -*-
"""
Class for transmitting learner data to SuccessFactors.
"""
from __future__ import absolute_import, unicode_literals
import logging
from django.apps import apps
from integrated_channels.sap_success_factors.transmitters import SuccessFactorsTransmitterBase
from requests import RequestException


LOGGER = logging.getLogger(__name__)


class SuccessFactorsLearnerDataTransmitter(SuccessFactorsTransmitterBase):
    """
    This endpoint is intended to receive learner data routed from the integrated_channel app that is ready to be
    sent to SuccessFactors.
    """

    def transmit(self, payload):
        """
        Send a completion status call to SAP SuccessFactors using the client.

        Args:
            payload (LearnerDataTransmissionAudit): The learner completion data payload to send to SAP SuccessFactors
        """
        serialized_payload = payload.serialize()
        LOGGER.info(serialized_payload)

        enterprise_enrollment_id = payload.enterprise_course_enrollment_id
        if payload.completed_timestamp is None:
            # The user has not completed the course, so we shouldn't send a completion status call
            LOGGER.debug('Skipping in progress enterprise enrollment {}'.format(enterprise_enrollment_id))
            return None

        LearnerDataTransmissionAudit = apps.get_model(  # pylint: disable=invalid-name
            app_label='sap_success_factors',
            model_name='LearnerDataTransmissionAudit'
        )

        previous_transmissions = LearnerDataTransmissionAudit.objects.filter(
            enterprise_course_enrollment_id=enterprise_enrollment_id,
            error_message=''
        )
        if previous_transmissions.exists():
            # We've already sent a completion status call for this enrollment
            LOGGER.debug('Skipping previously sent enterprise enrollment {}'.format(enterprise_enrollment_id))
            return None

        try:
            code, body = self.client.send_completion_status(payload.sapsf_user_id, serialized_payload)
            LOGGER.debug('Successfully sent completion status call for enterprise enrollment {} with payload {}'.
                         format(enterprise_enrollment_id, serialized_payload))
        except RequestException as request_exception:
            code = 500
            body = str(request_exception)
            LOGGER.error('Failed to send completion status call for enterprise enrollment {} with payload {}'
                         '\nError message: {}'.format(enterprise_enrollment_id, payload, body))

        payload.status = str(code)
        payload.error_message = body if code >= 400 else ''
        payload.save()
        return payload



# -*- coding: utf-8 -*-
"""
URLs for integrated_channel.
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from integrated_channels.integrated_channel.views import (PushCatalogDataToIntegratedChannel,
                                                          PushLearnerDataToIntegratedChannel)

urlpatterns = [
    url(
        r'^integrated_channel/push_catalog_data',
        PushCatalogDataToIntegratedChannel.as_view(),
        name='push_catalog_data'
    ),
    url(
        r'^integrated_channel/push_learner_data',
        PushLearnerDataToIntegratedChannel.as_view(),
        name='push_learner_data'
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0020_auto_20170624_2316'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalenrollmentnotificationemailtemplate',
            name='history_change_reason',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecourseenrollment',
            name='history_change_reason',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecustomer',
            name='history_change_reason',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecustomerentitlement',
            name='history_change_reason',
        ),
        migrations.RemoveField(
            model_name='historicaluserdatasharingconsentaudit',
            name='history_change_reason',
        ),
    ]



# -*- coding: utf-8 -*-
"""
Views for Enterprise Integrated Channel.
"""

from __future__ import absolute_import, unicode_literals
from rest_framework.views import APIView


class PushLearnerDataToIntegratedChannel(APIView):
    """
    Provide an endpoint to route learner data to the appropriate channels based on Enterprise's configuration.
    """

    def post(self, request):
        """
        Hit when certificates are generated or when audit learners in self paced courses reach a passing grade.

        When this happens, this method will route the learners data appropriately.

        For a given learner who has completed a course, find out the following:
            if the learner is associated with an enterprise customer.
            if the course is part of an enterprise enrollment.
            if the enterprise customer is configured for an Integrated Channel.

        Continue if all the conditions above are met, otherwise abort processing.

        Based on the EnterpriseCustomerPluginConfiguration for the Enterprise, determine which endpoints are
        configured to receive learner data, and send a formatted request to each of those endpoints.
        If nothing is configured, abort processing.
        Note that the request format will be uniform across the different plugin endpoints.
        """
        raise NotImplementedError('Will be implemented as part of ENT-193')


class PushCatalogDataToIntegratedChannel(APIView):
    """
    Provide an endpoint to export a course catalog to the appropriate channels based on an Enterprise's configuration.
    """

    def post(self, request):
        """
        Trigger sending an enterprise's course catalog data to any configured endpoints.

        For a given enterprise customer, based on the EnterpriseCustomerPluginConfiguration for the Enterprise,
        determine which plugin endpoints are configured to receive course data, and send a formatted request to
        each of those endpoints.
        If nothing is configured, abort processing.
        Note that the request format will be uniform across the different plugin endpoints.
        """
        raise NotImplementedError('Will be implemented as part of ENT-192')
